#!/bin/bash
#
##################
##### LiHAS ######
##################
#
# Paket: aptchk4u
#
# /usr/bin/aptchk4u.sh: Main Script
#
# Autoren: Thomas Hoth (Mail, Tmpfile, Ratschlaege),
#          Daniel Philipp (vserver Erweiterungen, Rewrite, Funktionen)
#          Johannes Schöpfer, umbauen um aptitude anstatt apt-get zu ebnutzen
#
# Beschreibung: Skript zur Ueberpruefung von vorhandenen Updates fuer Debian;
#               ueberprueft Hostsystem und virtuelle Server mit
#               anschliessendem Versenden einer E-Mail bei notwendigen
#               Updates
# letzte Aenderung: 2008-10-17-jsc


#### VARIABLEN

export LANG=C

HOSTNAME="$(hostname)"

# Programme, die nicht im Paket "coreutils" enthalten sind
AWK=awk
MUTT=mutt
SED=sed


# auf Vserver bezogene Variablen
VS_DIR=/var/lib/vservers
VS_EXE=vserver

# temporaere Datei (beinhaltet Namen der Upgrade-Pakete)
TMPFILE=$(mktemp /tmp/aptchk4u-tmp.XXXXXX)
TMPERRFILE=$(mktemp /tmp/aptchk4u-err.XXXXXX)

CONFIGFILE="/etc/aptchk4u/aptchk4u.rc"
MUTTCONFIG="/etc/aptchk4u/muttrc"

### FUNKTIONEN:

# vserver Hauptfunktion: diese Funktion fragt den Status der aktiven,
# virtuellen Server ab und aktualisiert dann mittels der 1.
# Unterfunktion die Paketlisten der einzelnen virtuellen Server;
# mittels der 2. Unterfunktion wird eine Liste der benoetigten Pakete
# erstellt und anschliessend eine E-Mail je virtuellem Server verschickt

function check_vs_run() {
  echo -e "running vservers:  "`vserver-stat | tail -n +2 | grep -v '^0 .*root server' | awk '{print $NF}'`"\n" >>$TMPFILE
  cd $VS_DIR
    ls | while read a; do
      vserver $a running 2> /dev/null
      if [ "$?" == 0 ]; then
        vs_agud $a
        check_vs_pack $a
      fi
    done
 }


function aptversion() {
  DISTRI=unkown
  debianv=0
  APT_G=false
  version=""
  if $vserver [ -e /etc/debian_version ]; then
    version=`$vserver cat /etc/debian_version`
    DISTRI=debian
  # Check if sarge or etch, for sarge use apt-get
    debianv=$($vserver cat /etc/debian_version | cut -d'.' -f1| cut -d'/' -f1)
    case "$debianv" in
    sarge) debianv=3;;
    etch)  debianv=4;;
    lenny) debianv=5;;
    [0-9]) true ;;
    *) debianv=6;;
    esac

    if [ "$debianv" -gt 3  ]
    then
       APT_G="aptitude -o Aptitude::CmdLine::Ignore-Trust-Violations=true"
    else
       APT_G=apt-get
    fi
    apt_p=$($vserver which ${APT_G%% *})
    $vserver [ -x $apt_p ] || APT_G=false
  elif  $vserver [ -e /etc/redhat-release ]; then
     DISTRI=redhat
     redhatv=`$vserver cat /etc/redhat-release`
     version=`$vserver cat /etc/redhat-release`
  else
     DISTRI=unkown
     debianv=0
     APT_G=false
  fi
}
  
# vserver 1. Unterfunktion: aktualisiert die Paketlisten der einzelnen
# virtuellen Server
  
function vs_agud() {
  local ERR=''
  vs_host="$1"
  vserver="$VS_EXE $vs_host exec" 
  aptversion
  # catch stderr into $ERR
  [ "$DISTRI" = "debian" ] && ERR=$( ( $vserver $APT_G update > /dev/null ) 2>&1 )
  [ -n "$ERR" ] && echo -e "vserver $vs_host($version) / update errors:\n$ERR\n" > $TMPERRFILE
}

# vserver 2. Unterfunktion: erstellt Liste der benoetigten Pakete und haengt
# diese an eine temporaere Datei an

function check_vs_pack() {
  VS_HOSTNAME=$a
  #if $VS_EXE $a exec [ -e /etc/hostname ]; then
  #  VS_HOSTNAME=$($VS_EXE $a exec cat /etc/hostname)
  #else
  #  VS_HOSTNAME=$a
  #fi
  case "$DISTRI" in 
  debian)
    if [ "$APT_G" == false ]; then
      echo -e "vserver $VS_HOSTNAME:   aptitude not found or not executable" >> $TMPFILE
    else
      PACKAGES=$($VS_EXE $a exec $APT_G -q -y -s dist-upgrade 2>>$TMPERRFILE | grep -v 'aptitude::CmdLine::Ignore-Trust-Violations' | grep ^\ )
      #if [ -n "$PACKAGES" ] ; then
        echo -e "vserver $VS_HOSTNAME($version):   $PACKAGES \n" >> $TMPFILE
	ERR=$( cat $TMPERRFILE ) 
	echo "$ERR" >>$TMPFILE
	echo  >>$TMPFILE
      #fi
    fi
    if [ "$debianv" == 0 ]; then
      echo -e "    missing /etc/debian_version\n" >> $TMPFILE
    fi
  ;;
  redhat)
    PACKAGES=$($VS_EXE $a exec  yum check-update | grep ' base \| updates ' | awk '{print $1}')
    echo -e "vserver $VS_HOSTNAME($version):   "$PACKAGES"\n" | fmt >> $TMPFILE
  ;;
  *)
    echo -e "vserver $VS_HOSTNAME: unkown distri\n" >>$TMPFILE 
  esac
}

# Host-Funktion: aktualisiert die Paketlisten des Hosts, erstellt eine Liste
# der benoetigten Pakete und haengt diese an eine temporaere Datei an

function check_host_pack() {
  vserver=
  aptversion
  $APT_G update > /dev/null 2>$TMPERRFILE
  PACKAGES=$($APT_G -q -y -s dist-upgrade 2>>$TMPERRFILE | grep ^\ )
  #if [ -n "$PACKAGES" ] ; then
    echo -e "$HOSTNAME($version):   $PACKAGES \n" >> $TMPFILE
    cat $TMPERRFILE >>$TMPFILE
    echo  >>$TMPFILE
  #fi
}

# Mailfunktion: verschickt den Inhalt der temporaeren Datei, wenn Inhalt > 0

function mail_tmpfile() {
  if [ -n "$(cat $TMPFILE)" ]; then
    cat $TMPFILE | $MUTT -F $MUTTCONFIG -s "$name_of_company: Update on host \"$HOSTNAME\" needed" $email_address
  else
    echo "temporary file not found or zero content" > /dev/null
  fi
}

###


### MAIN:

. $CONFIGFILE

# 1. Parameter ist E-Mail-Adresse, wenn vorhanden;
# ansonsten E-Mail-Adresse aus Konfigurationsdatei
if [ ! -z "$1" ] ; then
  email_address="$1"
fi

check_host_pack

# ueberpruefe Existenz des vserver Verzeichnissses unter /
# und fuehre erst dann die vserver Hauptfunktion aus

if [ -d "$VS_DIR" ]; then
  check_vs_run
else
  echo "vservers directory not found" > /dev/null
fi

mail_tmpfile

rm $TMPFILE
